﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace SelfReferenceAndHierarchicalData
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = new TreesViewModel();
        }
    }

    #region ViewModel

    public class TreesViewModel
    {
        public TreesViewModel()
        {
            SelfRef = TreeData.GenerateTreeData();
            Hierarchy = Root.GetData();
        }

        public ObservableCollection<TreeData> SelfRef { get; set; }
        public ObservableCollection<Root> Hierarchy { get; set; }
    }



    public class PropertyChangedNotifier : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    #endregion

    #region Hierarchy

    public class Root
    {
        public string Name { get; set; }
        public ObservableCollection<ChildLevel1> Children { get; set; }

        public static ObservableCollection<Root> GetData()
        { 
            ObservableCollection<Root> data = new ObservableCollection<Root>();

            for (int i = 0; i < 5; i++)
            {
                Root r = new Root { Name = "Root " + i, Children = new ObservableCollection<ChildLevel1>() };

                for (int j = 0; j < 5; j++)
                {
                    ChildLevel1 c1 = new ChildLevel1 { Name = "ChildLevel1 " + j, Children = new ObservableCollection<ChildLevel2>() };

                    for (int p = 0; p < 5; p++)
                    {
                        ChildLevel2 c2 = new ChildLevel2 { Name = "ChildLevel2 " + p, Children = new ObservableCollection<ChildLevel3>() };

                        for (int q = 0; q < 5; q++)
                        {
                            c2.Children.Add(new ChildLevel3 { Name = "ChildLevel3 " + q });
                        }

                        c1.Children.Add(c2);
                    }

                    r.Children.Add(c1);
                }

                data.Add(r);
            }

            return data;
        }
    }

    public class ChildLevel1
    {
        public string Name { get; set; }
        public ObservableCollection<ChildLevel2> Children { get; set; }
    }

    public class ChildLevel2
    {
        public string Name { get; set; }
        public ObservableCollection<ChildLevel3> Children { get; set; }
    }

    public class ChildLevel3
    {
        public string Name { get; set; }
    }

    #endregion

    #region Self Reference

    public class TreeData : PropertyChangedNotifier
    {
        private int id;

        public int ID
        {
            get { return id; }
            set { id = value; OnPropertyChagned("ID"); }
        }

        private string name;

        public string Name
        {
            get { return name; }
            set { name = value; OnPropertyChagned("Name"); }
        }

        private ObservableCollection<TreeData> children;

        public ObservableCollection<TreeData> Children
        {
            get { return children; }
            set { children = value; OnPropertyChagned("Children"); }
        }

        public static ObservableCollection<TreeData> GenerateTreeData()
        {
            ObservableCollection<TreeData> data = new ObservableCollection<TreeData>();

            for (int i = 0; i < 10; i++)
            {
                TreeData n = new TreeData
                {
                    ID = i,
                    Name = "Item " + i,
                    Children = new ObservableCollection<TreeData>()
                };

                for (int j = 0; j < 5; j++)
                {
                    TreeData n1 = new TreeData
                    {
                        ID = i + j,
                        Name = "Item " + i + " " + j,
                        Children = new ObservableCollection<TreeData>()
                    };

                    for (int p = 0; p < 5; p++)
                    {
                        n1.children.Add(new TreeData
                        {
                            ID = i + j + p,
                            Name = "Item " + i + " " + j + " " + p,
                        });
                    }

                    n.Children.Add(n1);
                }

                data.Add(n);
            }

            return data;
        }

    }

    #endregion

   
}
